<?php if ( __FILE__ == $_SERVER['SCRIPT_FILENAME'] ) { die(); }


// Execute hooks before framework loads
do_action( 'functions_before' );


#-----------------------------------------------------------------
# Load framework
#-----------------------------------------------------------------
include_once get_template_directory() . '/framework/load.php';



// Execute hooks after framework loads
do_action( 'functions_after' ); ?>
<?php

// Content width
if ( ! isset( $content_width ) ) $content_width = 783; 

// Template size variables
if ( ! isset( $max_content_width ) ) $max_content_width = 1200; // the largest possible width
if ( ! defined( 'MAX_COLUMNS' ) ) define( 'MAX_COLUMNS', 12); // number of columns in layout


#-----------------------------------------------------------------
# Theme Features
#-----------------------------------------------------------------

function theme_features_setup() {
	
	// Translation
	load_theme_textdomain( 'framework', get_template_directory() . '/languages' );
	
	// WP Stuff
	add_editor_style(); // Admin editor styles
	add_theme_support( 'automatic-feed-links' ); // RSS feeds	
	add_theme_support( 'post-formats', array( 'audio', 'gallery', 'image', 'link', 'quote', 'video' ) ); // Post formats.	
	register_nav_menu( 'primary', __( 'Primary Menu', 'framework' ) ); // Main menu

	// Image sizes
	add_theme_support( 'post-thumbnails' );
	set_post_thumbnail_size( 783, 9999 ); // default width, unlimited height, soft crop
	add_image_size( 'portfolio-thumb',  768, 432, true ); // 16x9 ratio, hard crop
	add_image_size( 'half-width-thumb', 575, 9999 );
	add_image_size( 'full-width-thumb', 1200, 9999 );
}
add_action( 'after_setup_theme', 'theme_features_setup' );


#-----------------------------------------------------------------
# Styles and Scripts
#-----------------------------------------------------------------

function theme_styles_and_scripts() {
	global $wp_styles;

	$layout = get_layout(); // Get theme layout data
	$layoutSuffix = $layout['alias'];
	// JavaScript
	//...............................................
	wp_deregister_script('jquery-ui-tabs');
	wp_enqueue_script('jquery-ui-tabs', get_stylesheet_directory_uri() . '/assets/js/jquery.ui.tabs.min.js', array('jquery-ui-core', 'jquery-ui-widget'), '1.10.4', 1 );

	// Register scripts
	wp_register_script( 'modernizr',  get_stylesheet_directory_uri() . '/assets/js/modernizr-2.6.2-respond-1.1.0.min.js', array(), '2.6.2' );
	wp_register_script( 'jplayer',    get_stylesheet_directory_uri() . '/assets/js/jquery.jplayer.min.js', array('jquery'), '2.2.0', true );
	wp_register_script( 'flexslider', get_stylesheet_directory_uri() . '/assets/js/jquery.flexslider-min.js', array('jquery'), '2.1', true );
	wp_register_script( 'rw_isotope',    get_stylesheet_directory_uri() . '/assets/js/jquery.isotope.min.js', array('jquery'), '1.5.25', true );
	wp_register_script( 'theme-js',   get_stylesheet_directory_uri() . '/assets/js/onLoad.js', array('jquery'), '1.0', true ); // The theme's JS functions

	// Enqueue scripts
	wp_enqueue_script( 'modernizr' ); 
	wp_enqueue_script( 'jplayer' );
	wp_enqueue_script( 'flexslider' ); 
	wp_enqueue_script( 'rw_isotope' );   // TODO: load only on filtered portfolio pages
	wp_enqueue_script( 'theme-js' );

	// Threaded comment support
	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) )
		wp_enqueue_script( 'comment-reply' );

	// CSS
	//...............................................

	// Default
	wp_enqueue_style( 'theme-css', get_stylesheet_uri() );

	// Skin
	$skin = (get_options_data('options-page', 'skin')) ? get_options_data('options-page', 'skin') : 'style-skin-1.css'; // default skin setting
	$skin = (get_options_data('other_options_' . $layoutSuffix, 'field-Y4p67yCm4P8e') != "") ? get_options_data('other_options_' . $layoutSuffix, 'field-Y4p67yCm4P8e') : $skin; // layout specific skin
	$skin = apply_filters( 'skin_css', $skin ); // filter for overloading (if needed)
	wp_enqueue_style( 'theme-skin', get_stylesheet_directory_uri() . '/'. $skin, array( 'theme-css' ) );

	// Feature specific
	$deps = array();
	if( wp_style_is( 'font-awesome', 'registered' ) )
		$deps[] = 'font-awesome';
	$deps[] = 'ubermenu-font-awesome';	
	wp_enqueue_style( 'font-awesome-theme', get_stylesheet_directory_uri() . '/assets/css/font-awesome.min.css', $deps );
	wp_enqueue_style( 'colorbox', get_stylesheet_directory_uri() . '/assets/css/colorbox.css' );
	// --- MOVED TO: override_vc_styles() ---
	// wp_enqueue_style( 'flexslider', get_stylesheet_directory_uri() . '/assets/css/flexslider.css', array(), '2.1' ); 

	// IE only
	wp_enqueue_style( 'theme-ie', get_stylesheet_directory_uri() . '/assets/css/ie.css', array( 'theme-css' ) );
		$wp_styles->add_data( 'theme-ie', 'conditional', 'lt IE 9' );
	wp_enqueue_style( 'font-awesome-ie', get_stylesheet_directory_uri() . '/assets/css/font-awesome-ie7.min.css', array( 'font-awesome' ) );
		$wp_styles->add_data( 'font-awesome-ie', 'conditional', 'lte IE 7' );

	// Fonts
	//...............................................

	// Include Google Fonts
	$googleFonts   = array();
	$gFont_Heading = get_options_data('options-page', 'font-heading-google');
	$gFont_Body    = get_options_data('options-page', 'font-body-google');

	if ( !empty($gFont_Heading) ) $googleFonts[] = $gFont_Heading;
	if ( !empty($gFont_Body) ) $googleFonts[] = $gFont_Body;

	if ( count($googleFonts) ) {  
		$gFontList  = str_replace(' ', '+', implode('|', $googleFonts)); // make ready for query string		
		$protocol   = is_ssl() ? 'https' : 'http';
		$subsets    = 'latin,latin-ext';
		$query_args = array( 'family' => $gFontList, 'subset' => $subsets );
		wp_enqueue_style( 'google-font', add_query_arg( $query_args, "$protocol://fonts.googleapis.com/css" ), array(), null );
	}
}
add_action( 'wp_enqueue_scripts', 'theme_styles_and_scripts' );

// Visual Composer specfic CSS files to remove or replace
function override_vc_styles() {
    wp_deregister_style( 'flexslider' ); // remove VC specific CSS
    wp_enqueue_style( 'flexslider', get_stylesheet_directory_uri() . '/assets/css/flexslider.css', array(), '2.1' ); // Add theme specific CSS
}
add_action( 'wp_enqueue_scripts', 'override_vc_styles', 99 );

// Add Runway credits
function built_with_runway() {
	echo '<style type="text/css">#footer-thankyou { display:none; } </style>';
	echo '<script>jQuery("p#footer-left").html(\'Built with <a href="http://runwaywp.com" target="_blank">Runway</a> for <a href="http://wordpress.org" target="_blank">WordPress</a>\');</script>';
}
add_action('admin_footer', 'built_with_runway');

// Custom Styles and Script
//-----------------------------------------------------------------
function theme_options_custom_js_and_css() {

	// Custom Styles from Theme Options
	echo '<style type="text/css">';
	theme_custom_styles();
	echo '</style>';

	// Custom Scripts from Theme Options
	echo '<script type="text/javascript">';
	theme_custom_scripts();
	echo '</script> ';

}
// Add hook for front-end <head></head>
add_action('wp_head', 'theme_options_custom_js_and_css');


#-----------------------------------------------------------------
# Site Title
#-----------------------------------------------------------------

function format_wp_title( $title, $sep ) {
	global $paged, $page;

	if ( is_feed() )
		return $title;

	// Add the site name.
	$title .= get_bloginfo( 'name' );

	// Add the site description for the home/front page.
	$site_description = get_bloginfo( 'description', 'display' );
	if ( $site_description && ( is_home() || is_front_page() ) )
		$title = "$title $sep $site_description";

	// Add a page number if necessary.
	$_paged = (isset($paged) && is_numeric($paged)) ? $paged : 0;
	$_page = (isset($page) && is_numeric($page)) ? $page : 0;
	$paging = max( intval($_paged), intval($_page) );
	if ( $paging >= 2 )
		$title = "$title $sep " . sprintf( __( 'Page %s', 'framework' ), $paging );

	return $title;
}
add_filter( 'wp_title', 'format_wp_title', 10, 2 );


#-----------------------------------------------------------------
# Menu (fallback to page list)
#-----------------------------------------------------------------

function theme_page_menu_args( $args ) {
	if ( ! isset( $args['show_home'] ) )
		$args['show_home'] = true;
	return $args;
}
add_filter( 'wp_page_menu_args', 'theme_page_menu_args' );


#-----------------------------------------------------------------
# Default Sidebars
#-----------------------------------------------------------------

function theme_default_sidebars() {
	// Default sidebar
	register_sidebar( array(
		'name' => __( 'Default Sidebar', 'framework' ),
		'id' => 'sidebar-default',
		'description' => __( 'The default sidebar.', 'framework' ),
		'before_widget' => '<aside id="%1$s" class="widget %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget-title">',
		'after_title' => '</h3>',
	) );
	// Footer Top
	register_sidebar( array(
		'name' => __( 'Footer - Top Section', 'framework' ),
		'id' => 'sidebar-footer-top',
		'description' => __( 'The default content of the footer top section. If your footer content is not coming from this location, look for it inside the Static Content (static blocks) menu of your admin.', 'framework' ),
		'before_widget' => '<aside id="%1$s" class="widget %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget-title">',
		'after_title' => '</h3>',
	) );
	// Footer Bottom
	register_sidebar( array(
		'name' => __( 'Footer - Bottom Section', 'framework' ),
		'id' => 'sidebar-footer-bottom',
		'description' => __( 'The default content of the footer bottom section. If your footer content is not coming from this location, look for it inside the Static Content (static blocks) menu of your admin.', 'framework' ),
		'before_widget' => '<aside id="%1$s" class="widget %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h3 class="widget-title">',
		'after_title' => '</h3>',
	) );
}
add_action( 'widgets_init', 'theme_default_sidebars' );


#-----------------------------------------------------------------
# Next / Previous Post Navigation
#-----------------------------------------------------------------
if ( ! function_exists( 'next_and_previous_post_navigation' ) ) :
/**
 * Displays navigation to next/previous pages when possible.
 */
function next_and_previous_post_navigation( $html_id ) {
	global $wp_query;

	$html_id = esc_attr( $html_id );

	if ( $wp_query->max_num_pages > 1 ) : ?>
		<nav id="<?php echo $html_id; ?>" class="navigation" role="navigation">
			<h3 class="assistive-text"><?php _e( 'Post navigation', 'framework' ); ?></h3>
			<div class="nav-previous alignleft"><?php next_posts_link( __( '<span class="meta-nav">&larr;</span> Older posts', 'framework' ) ); ?></div>
			<div class="nav-next alignright"><?php previous_posts_link( __( 'Newer posts <span class="meta-nav">&rarr;</span>', 'framework' ) ); ?></div>
		</nav><!-- #<?php echo $html_id; ?> .navigation -->
	<?php endif;
}
endif;


#-----------------------------------------------------------------
# Comments and Pingbacks
#-----------------------------------------------------------------

if ( ! function_exists( 'theme_comment' ) ) :
function theme_comment( $comment, $args, $depth ) {
	$GLOBALS['comment'] = $comment;
	switch ( $comment->comment_type ) :
		case 'pingback' :
		case 'trackback' :
		// Display trackbacks differently than normal comments.
	?>
	<li <?php comment_class(); ?> id="comment-<?php comment_ID(); ?>">
		<p><?php _e( 'Pingback:', 'framework' ); ?> <?php comment_author_link(); ?> <?php edit_comment_link( __( '(Edit)', 'framework' ), '<span class="edit-link">', '</span>' ); ?></p>
	<?php
			break;
		default :
		// Proceed with normal comments.
		global $post;
	?>
	<li <?php comment_class(); ?> id="li-comment-<?php comment_ID(); ?>">
		<article id="comment-<?php comment_ID(); ?>" class="comment">
			<header class="comment-meta comment-author vcard">
				<?php
					echo get_avatar( $comment, 44 );
					printf( '<cite class="fn">%1$s %2$s</cite>',
						get_comment_author_link(),
						// If current post author is also comment author, make it known visually.
						( $comment->user_id === $post->post_author ) ? '<span> ' . __( 'Post author', 'framework' ) . '</span>' : ''
					);
					printf( '<a href="%1$s"><time datetime="%2$s">%3$s</time></a>',
						esc_url( get_comment_link( $comment->comment_ID ) ),
						get_comment_time( 'c' ),
						/* translators: 1: date, 2: time */
						sprintf( __( '%1$s at %2$s', 'framework' ), get_comment_date(), get_comment_time() )
					);
				?>
			</header><!-- .comment-meta -->

			<?php if ( '0' == $comment->comment_approved ) : ?>
				<p class="comment-awaiting-moderation"><?php _e( 'Your comment is awaiting moderation.', 'framework' ); ?></p>
			<?php endif; ?>

			<section class="comment-content comment">
				<?php comment_text(); ?>
				<?php edit_comment_link( __( 'Edit', 'framework' ), '<p class="edit-link">', '</p>' ); ?>
			</section><!-- .comment-content -->

			<div class="reply">
				<?php comment_reply_link( array_merge( $args, array( 'reply_text' => __( 'Reply', 'framework' ), 'after' => ' <span>&darr;</span>', 'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) ); ?>
			</div><!-- .reply -->
		</article><!-- #comment-## -->
	<?php
		break;
	endswitch; // end comment_type check
}
endif;


#-----------------------------------------------------------------
# Post Meta Info (Item details: category, tags, author, etc...)
#-----------------------------------------------------------------

if ( ! function_exists( 'theme_post_meta' ) ) :
function theme_post_meta() {

	// Separator
	$sep = '<span class="meta-sep">/</span>';

	// Date
	$date = sprintf( '<span class="date-meta"><span class="meta-label">'. __( 'Date:', 'framework' ) .' </span><time class="entry-date" datetime="%1$s">%2$s</time>'.$sep.'</span>',
		esc_attr( get_the_date( 'c' ) ),
		esc_html( get_the_date() )
	);

	// Author
	$author = sprintf( '<span class="author vcard author-meta"><span class="meta-label">'. __( 'Author:', 'framework' ) .' </span><a class="url fn n" href="%1$s" title="%2$s" rel="author">%3$s</a>'. $sep .'</span>',
		esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ),
		esc_attr( sprintf( __( 'View all posts by %s', 'framework' ), get_the_author() ) ),
		get_the_author()
	);

	// Categories
	if ( get_post_type() == 'portfolio' ) {
		// Portfolio categories
		$categories_list = get_the_term_list( $post->ID, 'portfolio-category', '', __( ', ', 'framework' ) );
	} else {
		// Blog categories
		$categories_list = get_the_category_list( __( ', ', 'framework' ) );
	}

	if ($categories_list) 
		$categories = '<span class="categories-meta"><span class="meta-label">'. __( 'Category:', 'framework' ) .' </span>'. $categories_list .'</span>';

	// Comments
	$comments_link = '';
	if ( comments_open() ) {
		$count = get_comments_number();
		if ( $count == 0 ) {
			$comments = __( 'Leave a reply', 'framework' );
		} elseif ( $count > 1 ) {
			$comments = $count . __( ' Replies', 'framework' );
		} else {
			$comments = __( '1 Reply', 'framework' );
		}
		$show_sep = ($categories) ? $sep : '';
		$comments_link = '<span class="comments-meta">'. $show_sep .'<a href="' . get_comments_link() .'" class="link-to-comments">'. $comments.'</a></span>';
	}

	// Output the items above
	echo '<div class="post-meta clearfix">'. $comments_link . $date . $author . $categories . '</div>';
	
	// Tag List (this is last)
	if ( has_tag() ) echo '<div class="tags-meta"><span class="meta-label">'. __( 'Tags:', 'framework' ) .' </span>'. get_the_tag_list( '', __( ', ', 'framework' ) ) .'</div>';

}
endif;


#-----------------------------------------------------------------
# Body Class
#-----------------------------------------------------------------

function add_theme_body_class( $classes ) {

	// Get layout data
	$layout = get_layout();
	$layoutSuffix = $layout['alias'];
	// Layout Specific
	//...............................................

	if ( $layout && isset($layout['title']) ) {
		$classes[] = 'layout-'. sanitize_title($layout['title']);
	} else {
		// If not using Layout Manager (or if error, fallback is default WP functionality)
		$classes[] = 'no-layout';
		// More cases specific no Layout Manager
		if ( ! is_active_sidebar( 'sidebar-default' ) || is_page_template( 'templates/full-width.php' ) || is_page_template( 'templates/portfolio-filtered.php' ) || is_page_template( 'templates/portfolio-static.php' ) || is_tax('portfolio-category') || is_singular('portfolio')  )
			$classes[] = 'full-width';
	}

	// Layout Style
	$layout_style = (get_options_data('other_options_' . $layoutSuffix, 'field-lidFt9zyeSTB') != "") ? get_options_data('other_options_' . $layoutSuffix, 'field-lidFt9zyeSTB') : get_options_data('options-page', 'layout-style');

	if ($layout_style == 'boxed') 
		$classes[] = 'boxed';

	// context
	if ( isset($GLOBALS['context']) ) 
		$classes[] = 'context-'.$GLOBALS['context'];

	// Generic WP
	//...............................................

	// Post thumbnails
	if ( has_post_thumbnail() )
		$classes[] = 'has-post-thumbnail';

	// No author names on posts when site has only one author
	if ( ! is_multi_author() )
		$classes[] = 'single-author';

	return $classes;
}
add_filter( 'body_class', 'add_theme_body_class' );



#-----------------------------------------------------------------
# Include skins in Theme Options
#-----------------------------------------------------------------

if (is_admin() && !function_exists('theme_option_skin_select')) :
	function theme_option_skin_select( $options ) {
		
		$RunwayAdmin = new Runway_Admin_Object( array() );
		$skins = $RunwayAdmin->get_skin_css();

		$options = array('' => '');
		if (is_array($skins)) {
			foreach($skins as $key => $val)
				$options[$key] = $val;
		}

		return $options;
	}
	// add filter: [field alias]_data_options
	add_filter( 'skin_data_options', 'theme_option_skin_select' );
	add_filter( 'field-Y4p67yCm4P8e_data_options', 'theme_option_skin_select' );
endif;


#-----------------------------------------------------------------
# Add blog templates to blog context
#-----------------------------------------------------------------

if ( ! function_exists( 'blog_template_file_context' ) ) :
	function blog_template_file_context( $context, $template ) {

		$blog_template[] = locate_template('templates/blog-image-top.php');
		$blog_template[] = locate_template('templates/blog-image-left.php');

		if ( in_array($template, $blog_template) ) {
			$context = 'blog';
		}

		return $context;
	}
endif;



add_filter( 'template_context', 'blog_template_file_context', 10, 2 );


#-----------------------------------------------------------------
# Post Content and Related Functions
#-----------------------------------------------------------------

// Get image size information
//...............................................

if ( ! function_exists( 'get_post_image_size' ) ) :
	function get_post_image_size( $default = 'post-thumbnail' ) {

		$size = $default;

		// Image size
		$prefix = (is_single()) ? 'single-' : '';
		$img_w = get_options_data('blog-options', $prefix.'image-width');  
		$img_h = get_options_data('blog-options', $prefix.'image-height'); // defaults to 9999 for unlimited

		// is a size set for width or height
		if ( !empty($img_w) || !empty($img_h) ) {
			// defaults to 9999 for unlimited
			if (empty($img_w)) $img_w = 9999;
			if (empty($img_h)) $img_h = 9999;
			// set size
			$size = array($img_w, $img_h);
		}

		return $size;
	}
endif;

// Show Post Content (excerpt or full post)
//...............................................

if ( ! function_exists( 'theme_post_content' ) ) :
	function theme_post_content( $post_id = false, $content = false, $excerpt = false ) {
		global $post, $custom_query;

		$post_id = ($post_id) ? $post_id : $post->ID;

		$needExcerpts = (isset($custom_query))? true : false;

		$postExcerpts  = get_options_data('blog-options', 'post-excerpts', false);
		$excerptLength = get_options_data('blog-options', 'excerpt-length', false);
		$excerptLength = ( isset($custom_query->query) && isset($custom_query->query['excerpt_length']) && $custom_query->query['excerpt_length'] ) ? $custom_query->query['excerpt_length'] : $excerptLength;
		$readMore = get_options_data('blog-options', 'read-more', -1);
		$readMore_Exclude = array('quote','link','image'); // post formats to exclude read more link

		if ( (!is_single() || $needExcerpts) && ($postExcerpts || is_search()) ) : ?>
			<div class="entry-content summary">
				<p><?php echo customExcerpt(get_the_excerpt(), $excerptLength); ?></p>
				<?php if ( $readMore != -1 && !in_array(get_post_format(), $readMore_Exclude) ) : ?>
					<p class="readmore"><a href="<?php the_permalink() ?>"><?php echo $readMore ?></a></p>
				<?php endif; ?>
			</div><!-- .entry-summary -->
			<?php 
		else : ?>
			<div class="entry-content">
				<?php the_content( __( 'Continue reading <span class="meta-nav">&rarr;</span>', 'framework' ) ); ?>
				<?php wp_link_pages( array( 'before' => '<div class="page-links">' . __( 'Pages:', 'framework' ), 'after' => '</div>' ) ); ?>
			</div><!-- .entry-content -->
			<?php 
		endif;

	}
endif;


#-----------------------------------------------------------------
# Title Functions
#-----------------------------------------------------------------

// Show Page Titles
//...............................................
/**
 * Outputs title with link or static container for loops and single posts/pages
 */
if ( ! function_exists( 'theme_post_title' ) ) :
	function theme_post_title( $post_id = false, $tag = false, $title = false ) {
		global $post, $custom_query;

		$post_id   = ($post_id) ? $post_id : $post->ID;
		$tag       = ($tag) ? $tag : 'h1';
		$title     = ($title) ? $title : get_the_title($post_id); // esc_attr($post->post_title)
		$hideTitle = get_post_meta($post_id, 'hide_title', true);
		
		$needExcerpts = (isset($custom_query))? true : false;

		if ($hideTitle && is_single()) {
			return false;
		}

		if ( is_single() && !$needExcerpts ) :
			// single, get as just text
			echo '<'.$tag.' class="page-title">'. $title .'</'.$tag.'>';
		else :
			// list (blog) get as link
			echo '<'.$tag.' class="entry-title">';
			echo '	<a href="'. get_permalink($post_id) .'" title="'. sprintf( __( 'Permalink to %s', 'framework' ), the_title_attribute( 'echo=0' ) ) .'" rel="bookmark">'. $title .'</a>';
			echo '</'.$tag.'>';
		endif; // is_single() 

	}
endif;


// More Page Titles (outside the loop)
//...............................................
/**
 * Similar to titles generaged by wp_title() for use in headers and other areas outside the loop.
 */

function generate_title() {
	global $wpdb, $wp_locale;

	$m        = get_query_var('m');
	$year     = get_query_var('year');
	$monthnum = get_query_var('monthnum');
	$day      = get_query_var('day');
	$search   = get_search_query();
	$title    = '';
	$t_sep    = ' ';

	// If there is a post
	if ( is_single() || ( is_home() && !is_front_page() ) || ( is_page() && !is_front_page() ) ) {
		$title = single_post_title( '', false );
	}
	// If there's a category or tag
	if ( is_category() || is_tag() ) {
		$title = single_term_title( '', false );
	}
	// If there's a taxonomy
	if ( is_tax() ) {
		$term = get_queried_object();
		$tax = get_taxonomy( $term->taxonomy );
		$title = single_term_title( $tax->labels->name . $t_sep, false );
	}
	// If there's an author
	if ( is_author() ) {
		$author = get_queried_object();
		$title = $author->display_name;
	}
	// If there's a post type archive
	if ( is_post_type_archive() )
		$title = post_type_archive_title( '', false );
	// If there's a month
	if ( is_archive() && !empty($m) ) {
		$my_year = substr($m, 0, 4);
		$my_month = $wp_locale->get_month(substr($m, 4, 2));
		$my_day = intval(substr($m, 6, 2));
		$title = ( $my_month ? $my_month .  $t_sep : '' ) . ( $my_day ? $my_day . $t_sep : '' ) . $my_year;
	}
	// If there's a year
	if ( is_archive() && !empty($year) ) {
		$title = '';
		if ( !empty($monthnum) )
			$title .= $wp_locale->get_month($monthnum) . $t_sep;
		if ( !empty($day) )
			$title .= zeroise($day, 2) . $t_sep;
		$title .= $year;
	}
	// If it's a search
	if ( is_search() ) {
		/* translators: 1: separator, 2: search phrase */
		$title = sprintf(__('Search Results %1$s %2$s', 'framework'), __('for', 'framework'), strip_tags($search));
	}
	// If it's a 404 page
	if ( is_404() ) {
		$title = __('Page not found', 'framework');
	}

	return $title;

}


// Shortcode to display the page/post title. 
//...............................................
/**
 * 
 * [page_title container="h1" class="page-title"]
 * 
 */
if ( ! function_exists( 'theme_shortcode_post_title' ) ) :
	function theme_shortcode_post_title( $atts, $content = null ) {
		extract(shortcode_atts(array(
			'container' => 'h1',
			'class' => 'page-title'
	    ), $atts));
		
		$title = generate_title();
		if ( $container && $container != '0' && $container != 'false' ) {
			$title = '<'. $container .' class="'. $class .'" title="'. $title . '">'. $title .'</'. $container .'>';
		}
		
		return $title;
		
	}
	add_shortcode('page_title', 'theme_shortcode_post_title');
endif;

function ubermenu_custom_menu_item_defaults_lite( $defaults ){
	$cdefs = get_option( UBERMENU_MENU_ITEM_DEFAULTS_OPTION_KEY );
	if( $cdefs ){
		if( is_array( $cdefs ) ){
			foreach( $cdefs as $key => $val ){
				$defaults[$key] = $val;
			}
		}
	}
	$defaults['item_display'] = 'auto';
	return $defaults;
}
add_filter( 'ubermenu_menu_item_settings_defaults' , 'ubermenu_custom_menu_item_defaults_lite' );
