<?php global $custom_query, $wp_query, $portfolio; 

// Get shortcode params from the query_vars array
$params = ( isset($custom_query->query_vars) ) ? $custom_query->query_vars : false;

// Layout variables using params array passed from shortcode
$columns = (isset($params['columns']) && $params['columns']) ? $params['columns'] : 3; // number of columns (default 3)
$margin  = (isset($params['margin']) && $params['margin']) ? $params['margin'] : 30;   // Margin size (default 3 0px) [Future update: consider using %]
$width   = 1200; // only matters if we specify margin in pixels

// Do some calculations for column sizes
$portfolio['margin_width'] = round($margin/$width*100, 5, PHP_ROUND_HALF_DOWN); // % based margin (to 5 decimal places)
$portfolio['column_width'] = round(( 100 - ($portfolio['margin_width'] * ($columns-1)) ) / $columns , 5, PHP_ROUND_HALF_DOWN); // % based column

// Different margins for filtered portfolios 
if ($portfolio['type'] == 'filtered') {
	// Using 99.5% width becasue filtered container has overflow hidden. Maxing out will cut off shadows.
	$portfolio['column_width'] = round((99.5 - ($portfolio['margin_width'] * ($columns)) ) / $columns , 5, PHP_ROUND_HALF_DOWN); // % based column
}

// Check for a custom query, typically sent by a shortcode
$the_query = (!$custom_query) ? $wp_query : $custom_query;	
if ( $the_query->have_posts() ) : ?>

	<div class="row-fluid" style="clear: both;">

		<?php 
		// Show filters
		if ($portfolio['type'] == 'filtered') : 
			// specify width for main container for filters to align on left side
			$container_styles = "width: 90%; margin-left: 0px;";
			// list the filters ?>
			<div class="span-custom" style="width: 9.9%;">
				<ul id="sort-by" class="entry-meta">
					<li><a href="#all" data-filter="type-portfolio" class="active"><?php _e('All', 'framework'); ?></a></li>
					<?php wp_list_categories( array('title_li' => '', 'taxonomy' => 'portfolio-category', 'walker' => new Portfolio_Category_Walker() ) ); ?>
				</ul>
			</div>
			<div class="span-custom portfolio-wrapper isotope no-transition" style="<?php echo $container_styles; ?>">
			<?php
		endif; // end show filters ?>

			<?php

			// Show Portfolio List
			// ------------------------------------------------------------------
			$item_count = 0; // to track the loops

			// Loop through the results and print each. 
			while ( $the_query->have_posts() ) : $the_query->the_post();  
				
				// Get associated categories
				$terms =  get_the_terms( $post->ID, 'portfolio-category' ); 
				$term_list = '';
				if( is_array($terms) ) {
					foreach( $terms as $term ) {
						$term_list .= urldecode($term->slug) . ' ';
					}
				}

				// Apply the custom grid settings
				$classes = 'portfolio-item '. $term_list;

				// Filtered portfolio settings
				if ($portfolio['type'] == 'filtered') : 
					
					// data used by filter (isotope), helps with responsive adjustments
					$styles  = 'width: '. $portfolio['column_width'] .'%; margin-left: '. $portfolio['margin_width'] .'%;';
					$itemStyle_data = 'data-style=\'{"width": "'. $portfolio['column_width'] .'%", "margin": "'. $portfolio['margin_width'] .'%"}\'';
					$classes       .= 'isotope-item '; // add selector class
				
				// Static portfolio settings
				else :

					// custom class and styles for static portfolio
					$styles  = 'width: '. $portfolio['column_width'] .'%;';
					$itemStyle_data = '';
					$classes .= 'span-custom '; // add column class

					// margins and rows 
					if ($item_count > 0) {
						// first item in row doesn't get margin
						if (($item_count % $columns)) { 
							// not the first item, add left margin
							$styles .= ' margin-left: '. $portfolio['margin_width'] .'%;';
						} else {
							// new container for each row (end previous, start new)
							echo '</div><!-- .row-fluid -->';
							echo '<div class="row-fluid">';
						}
					}
				endif; // $portfolio['type'] == 'filtered' ?>

				<article id="post-<?php the_ID(); ?>" <?php post_class($classes); ?> style="<?php echo $styles; ?>" <?php echo $itemStyle_data; ?>>

					<header class="post-header">
						<?php 

						// Post formats that use the featured image (standard, image, audio)
						// --------------------------------------------------------------------
						$no_thumbnail = array('gallery', 'video');
						$media = get_the_post_thumbnail($post->ID, 'portfolio-thumb');
						$link_to_details  = get_permalink();
						if ( $media && !in_array(get_post_format(), $no_thumbnail) ) : 
							// A few items we need to create the media items
							$link_class       = 'shadow-curl styled-image '. get_post_format();
							$link_title       = sprintf( __( 'Permalink to %s', 'framework' ), the_title_attribute( 'echo=0' ) );
							$link_to_lightbox = wp_get_attachment_image_src( get_post_thumbnail_id($post->ID), 'full-width-thumb' );
							?>
							<div class="featured-image">
								<a href="<?php echo $link_to_details ?>" class="<?php echo $link_class ?>" title="<?php echo $link_title ?>" rel="bookmark"><?php echo $media ?><div class="inner-overlay"></div></a>
							</div>
							<?php 
						endif;

						// Media selected by post format
						switch( get_post_format() ) {

							case "audio" :
								// Audio Player
								theme_audio_player($post->ID);
								break;
							case "gallery" :
								?>
								<div class="featured-image">
									<div class="shadow-curl styled-image <?php echo get_post_format() ?>"><?php theme_gallery_slideshow($post->ID, false, 'portfolio-thumb'); ?></div>
								</div>
								<?php
								break;
							case "video" :
								// Video Player or Embed
								theme_video_player($post->ID);
								break;
						} ?>

					</header>

					<?php 

					// Item title
					theme_post_title( $post->ID, 'h2'); 

					// Item Content ?>
					<div class="entry-summary">
					<?php 
					// check for enabled excerpts on shortcode or always show excerpt on page template
					if ( !isset($params['template']) || (isset($params['post_excerpts']) && $params['post_excerpts']=='true') ) {
						// Check for a custom excerpt length
						if (isset($params['excerpt_length']) && !empty($params['excerpt_length'])) {
							echo customExcerpt(get_the_excerpt(), $params['excerpt_length']);
						} else {
							the_excerpt();
						}
					} ?>
					</div><!-- .entry-summary -->


					<footer class="entry-footer entry-meta">
						<span class="entry-portfolio-category"><?php the_terms($post->ID, 'portfolio-category', '', ', ', ''); ?></span>
					</footer><!-- .entry-meta -->

				</article><!-- #post -->

				<?php 

				$item_count++;

			endwhile;

		// end filtered portfolio wrapper
		if ($portfolio['type'] == 'filtered') { 
			echo '</div><!-- .portfolio-wrapper .isotope -->'; 
		} 

		?>

	</div><!-- .row-fluid -->

	<?php

	// Pagination
	$paging = ( isset($params['paging']) && $params['paging'] == 'false' ) ? false : true;
	if (  $paging ) : get_pagination($the_query); endif;


endif; // end have_posts() check 

?>